@@ -1,4 +1,3 @@
-use std::cmp::Ordering;
 use std::convert::TryInto;
 use std::fmt::{Display, Formatter};
 use std::time::{Duration, SystemTime, UNIX_EPOCH};
@@ -183,14 +182,6 @@ macro_rules! impl_timestamp {
         Self(signed)
       }
 
-      fn num_eq(&self, other: &Self) -> bool {
-        self.0.eq(&other.0)
-      }
-
-      fn num_cmp(&self, other: &Self) -> Ordering {
-        self.0.cmp(&other.0)
-      }
-
       fn to_bytes(self) -> Vec<u8> {
         ((self.0 - Self::MIN) as u128).to_be_bytes()[4..].to_vec()
       }
